package com.qmth.themis.business.service.impl;

import com.qmth.themis.business.constant.SystemConstant;
import com.qmth.themis.business.enums.MqEnum;
import com.qmth.themis.business.util.RedisUtil;
import com.qmth.themis.business.dto.MqDto;
import com.qmth.themis.business.service.MqDtoService;
import com.qmth.themis.business.service.ProducerServer;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;
import java.util.Objects;

/**
 * @Description: mqdto 服务实现类
 * @Param:
 * @return:
 * @Author: wangliang
 * @Date: 2020/7/2
 */
@Service
public class MqDtoServiceImpl implements MqDtoService {

    @Resource
    ProducerServer producerServer;

    @Resource
    RedisUtil redisUtil;

    /**
     * 组装单向消息
     *
     * @param o
     * @return
     */
    @Override
    public MqDto assembleSendOneWayMsg(Object... o) {
        MqDto mqDto = null;
        MqEnum mqEnum = MqEnum.valueOf(String.valueOf(o[3]));
//        for (int i = 0; i < 100; i++) {
//            int random = (int) (Math.random() * Source.values().length);
        //往mq发送消息插入会话信息
        mqDto = new MqDto(String.valueOf(o[0]), String.valueOf(o[1]), o[2], mqEnum, String.valueOf(o[4]), String.valueOf(o[5]));
//            mqDto = new MqDto(String.valueOf(o[0]), Source.values()[random].name(), o[2], mqEnum, String.valueOf(o[4]), String.valueOf(o[5]));
//            mqDto.setSequence(i);
        mqDto.setAck(SystemConstant.DELIVERED_ACK_TYPE);
//            producerServer.sendOneWay(mqDto);
//            switch (mqEnum.ordinal()) {
//                case 0:
//                    redisUtil.setSessionTopicList(SystemConstant.SESSION_TOPIC_BUFFER_LIST, mqDto.getId(), mqDto);
//                    break;
//                default:
//                    redisUtil.setSessionTopicList(SystemConstant.USERLOG_TOPIC_BUFFER_LIST, mqDto.getId(), mqDto);
//                    break;
//            }
//        }
        try {
            producerServer.sendOneWay(mqDto);
        } catch (Exception e) {
            e.printStackTrace();
            if (Objects.nonNull(mqDto)) {
                mqDto.setAck(SystemConstant.UNSEND_ACK_TYPE);
            }
        } finally {
            setTopicBuffer(mqDto);
        }
        return mqDto;
    }

    /**
     * 组装单向消息
     *
     * @param mqDto
     * @return
     */
    @Override
    public MqDto assembleSendOneWayMsg(MqDto mqDto) {
        mqDto.setAck(SystemConstant.DELIVERED_ACK_TYPE);
        try {
            producerServer.sendOneWay(mqDto);
        } catch (Exception e) {
            e.printStackTrace();
            if (Objects.nonNull(mqDto)) {
                mqDto.setAck(SystemConstant.UNSEND_ACK_TYPE);
            }
        } finally {
            setTopicBuffer(mqDto);
        }
        return null;
    }

    /**
     * 组装异步延时消息
     *
     * @param mqDto
     * @return
     */
    @Override
    public MqDto assembleSendAsyncDelayMsg(MqDto mqDto) {
        mqDto.setAck(SystemConstant.DELIVERED_ACK_TYPE);
        try {
            producerServer.asyncDelayMsg(mqDto);
        } catch (Exception e) {
            e.printStackTrace();
            if (Objects.nonNull(mqDto)) {
                mqDto.setAck(SystemConstant.UNSEND_ACK_TYPE);
            }
        } finally {
            setTopicBuffer(mqDto);
        }
        return null;
    }

    /**
     * 组装同步延时消息
     *
     * @param mqDto
     * @return
     */
    @Override
    public MqDto assembleSendSyncDelayMsg(MqDto mqDto) {
        mqDto.setAck(SystemConstant.DELIVERED_ACK_TYPE);
        try {
            producerServer.syncDelayMsg(mqDto);
        } catch (Exception e) {
            e.printStackTrace();
            if (Objects.nonNull(mqDto)) {
                mqDto.setAck(SystemConstant.UNSEND_ACK_TYPE);
            }
        } finally {
            setTopicBuffer(mqDto);
        }
        return null;
    }

    /**
     * 组装事务消息
     *
     * @param mqDto
     * @return
     */
    @Override
    public MqDto assembleSendTranMsg(MqDto mqDto) {
        mqDto.setAck(SystemConstant.DELIVERED_ACK_TYPE);
        try {
            producerServer.sendMsgTran(mqDto);
        } catch (Exception e) {
            e.printStackTrace();
            if (Objects.nonNull(mqDto)) {
                mqDto.setAck(SystemConstant.UNSEND_ACK_TYPE);
            }
        } finally {
            setTopicBuffer(mqDto);
        }
        return null;
    }

    /**
     * 设置mq buffer type
     *
     * @param mqDto
     */
    void setTopicBuffer(MqDto mqDto) {
        if (Objects.nonNull(mqDto)) {
            if (mqDto.getType().getId() < 100) {
                redisUtil.set(SystemConstant.MQ_TOPIC_BUFFER_LIST, mqDto.getId(), mqDto);
            } else {
                redisUtil.set(SystemConstant.MQ_DELAY_TOPIC_BUFFER_LIST, mqDto.getId(), mqDto);
            }
        }
    }
}
